window.siteimprove = window.siteimprove || {};
window.siteimprove.aem = window.siteimprove.aem || (function (window, $, logger) {
    'use strict';
    var config = {},
        overlayInserted = false,
        overlayDisabled = false,
        /**
         *
         * @param method the http Method to use
         * @param url the url to request
         * @param [data] the data used in the ajax request
         * @param [contentType] the content type of the payload sent to the server
         * @param [headers] the http headers to add to the request
         * @returns {JQuery.jqXHR}
         */
        ajax = function (method, url, data, contentType, headers) {
            var options = {
                method: method,
                url: url,
                dataType: 'json',
                cache: false
            };

            if (data) {
                options.data = data;
            }

            if (contentType) {
                options.contentType = contentType;
            }

            if (headers) {
                options.headers = headers;
            }

            return $.ajax(options);
        },
        /**
         * Adds customisations when using AEM in 'classic' aka ExtJS mode.
         */
        addClassicCustomisations = function () {
            // classic ui /siteadmin watch for changes
            if (window.location.pathname.indexOf("/siteadmin") === 0) {
                $(window).on('hashchange', function () {
                    getConfig().then(callWithConfig);
                });
            }
        },
        /**
         * Adds customisations when using AEM in 'touch ui' aka CoralUI mode.
         */
        addTouchUICustomisations = function () {
            logger.debug("Adding touch UI customizations");
            // supports navigating with tree view, column view, list view and card view
            // the issue is: when we select an item the navigation event fires
            // after this event the selection change event fires
            // this interferes with the pending request for the event from navigation
            $(document).on("foundation-collection-navigate", ".cq-siteadmin-admin-childpages", function(event) {
                if (event.target.classList.contains('granite-collection-loading-title-wrapper')) {
                    // event is fired twice in AEM card view, we discard this specific event.target
                    return;
                }

                // first navigate, afterwards selection change
                var $collection = $(".cq-siteadmin-admin-childpages.foundation-collection"),
                    pathForSiteimprove = $collection.data("foundationCollectionId");
                logger.debug("Handling foundation-collection-navigate");
                getConfig(pathForSiteimprove).then(callWithConfig);
            });


            // selection in card view
            $(document).on("foundation-selections-change", ".cq-siteadmin-admin-childpages", function(event) {
                var $selection = $(event.target).find(".foundation-selections-item");

                // for single selection we display the report for the selected item
                if ($selection.length === 1) {
                    logger.debug("Handling foundation-selections-change single item");
                    getConfig($selection.data("foundationCollectionItemId")).then(callWithConfig);
                } else { // fall back to parent page
                    logger.debug("Handling foundation-selections-change multiple items using parent");
                    getConfig($(event.target).data("foundationCollectionId")).then(callWithConfig);
                }
            });

        },
        addCustomisations = function () {
            addClassicCustomisations();
            addTouchUICustomisations();
        },
        addLoadHandler = function (handler) {
            logger.info('Adding load handler');
            if (window.addEventListener) {
                window.addEventListener('load', handler, false);
            } else if (window.attachEvent) {
                window.attachEvent('onload', handler);
            } else {
                window.onload = handler;
            }
        },
        getPath = function () {
            return window.location.pathname.split('?')[0] + window.location.hash.split('?')[0];
        },
        configCache = {},
        /**
         *
         * @param [pathToUse] optional path to use for retrieving the appropriate cloud configuration. If no path is given the current path is retrieved by calling getPath()
         * @returns {JQuery.Promise<any, any, any>}
         */
        getConfig = function (pathToUse) {
            if (pathToUse && pathToUse.length && configCache.hasOwnProperty(pathToUse)) {
                logger.trace("Using cached config for path", pathToUse)
                return configCache[pathToUse];
            }

            logger.info("Get config for", pathToUse);
            var d = $.Deferred(),
                path = pathToUse || getPath();

            configCache[pathToUse] = d;

            if (window.siteimprove.config) {
                // have a global config supplied somewhere
                logger.info("Using configuration from window variable");
                config = window.siteimprove.config;
                d.resolve(config);
            } else if (config.requestPath && config.requestPath === path) {
                // no change between path in config and currentPath
                logger.info("Using cached config as there has been no path change");
                d.resolve(config);
            } else {
                logger.info("Refreshing configuration for path", path);
                // refresh configuration
                var data = {
                    path: path
                };

                // TODO: configuration for this
                ajax('GET', '/conf/global/settings/siteimprove/jcr:content.config.json', data)
                    .done(function (configResponse) {
                        logger.info('Got configuration response for path', path);
                        config = configResponse;

                        overlayDisabled = !config.publicUrl && !config.domain;

                        logger.info('Overlay disabled?', overlayDisabled);

                        if (!overlayDisabled) {
                            $(document).find(".si-toggle-container").show();
                        }

                        d.resolve(config);
                    })
                    .fail(function (error) {
                        d.reject('Error retrieving config ' + error);
                    });
            }
            return d.promise();
        },
        insertJs = function(jsSrc) {
            var s = document.getElementsByTagName('script')[0],
                po = document.createElement('script');
            po.type = 'text/javascript';
            //po.async = true;
            po.src = jsSrc;
            s.parentNode.insertBefore(po, s);
        },
        /**
         *
         * @param config
         * @returns {JQuery.Promise<any, any, any>}
         */
        insertOverlay = function (config) {
            var overlaySrc = config.overlaySrc,
                d = $.Deferred();
            if (overlayInserted) {
                return d.resolve();
            }
            if(!config.publicUrl && !config.domain) {
                overlayDisabled = true;
            }

            if (!overlaySrc) {
                d.reject('Empty overlay src!');
            } else {
                logger.info('Inserting overlay ' + overlaySrc);
                insertJs(overlaySrc);
                overlayInserted = true;
                d.resolve();
            }
            return d.promise();
        },
        callSiteimprove = function (operation, domain, token) {
            var _si = window._si || [];

            logger.info(
                'Calling siteimprove with operation "' + operation + '", path "' + domain + '" and token "' + token + '".');
            _si.push([operation, domain, token, function () {
                logger.info('Called siteimprove with operation "' + operation + '" and path "' + domain + '".');
            }]);
        },
        clear = function () {
            callSiteimprove('clear', '', config.token);
        },
        input = function (domainAndPath) {
            callSiteimprove('input', domainAndPath, config.token);
        },
        domain = function (domain) {
            callSiteimprove('domain', domain, config.token);
        },
        recheck = function (domainAndPath) {
            callSiteimprove('recheck', domainAndPath, config.token);
        },
        recrawl = function (domain) {
            callSiteimprove('recrawl', domain, config.token);
        },
        /**
         * @param config The siteimprove configuration object, see SiteimproveContextConfiguration.
         * @param config.externalContentPath The external path of the current object as defined by mappings assigned in the cloud
         *     config.
         * @param config.publicUrl the full url of the page on publish
         * @param config.domain the domain of the public facing website
         * @param config.token the siteimprove token to be used for all calls
         */
        callWithConfig = function (config) {
            if (overlayDisabled) {
                logger.debug("Overlay is disabled, hiding si-toggle-container");
                $(document).find(".si-toggle-container").hide();
                return;
            } else {
                $(document).find(".si-toggle-container").show();
            }
            var operation = config.externalContentPath.length ? 'input' : 'domain',
                path = config.externalContentPath.length ? config.publicUrl : config.domain;
            callSiteimprove(operation, path, config.token);
        },
        /**
         *
         * @param {Document} contentDom
         * @param {Function} contentCheckCallback
         * @param {Function} highlightCallback
         * @param {string} [externalPath]
         */
        siCheckFlatDom = function(contentDom, contentCheckCallback, highlightCallback, externalPath) {
            if (overlayDisabled) {
                return;
            }
            var _si = window._si || [];
            _si.push(['onHighlight', highlightCallback]);
            getConfig().then(function (config) {
                var url = externalPath || config.externalContentPath.length ? config.publicUrl : '';
                logger.info("Running content check dom request for url", url);
                _si.push(['contentcheck-flat-dom', contentDom, url, config.token, contentCheckCallback]);
            });
        },
        init = function () {
            logger.info("siteimprove plugin init");
            if (window.siteimprove.initCalled
                || top.window.siteimprove && top.window.siteimprove.initCalled
                || top.window instanceof Array && top.window[0] && top.window[0].siteimprove && top.window[0].siteimprove.initCalled) {
                logger.trace("Init has been called already, ignoring subsequent init call.");
                return;
            }
            window.siteimprove.initCalled = true;

            getConfig()
                .done(insertOverlay)
                .then(function () {
                    addLoadHandler(function () {
                        callWithConfig(config);
                        addCustomisations();
                    });
                });
        };

    // public functions
    return {
        init: init,
        getConfig: getConfig,
        // supports supplying all parameters
        callSiteimprove: callSiteimprove,
        // Page Content Context (known page/content url)
        input: input,
        // Non Content Context (Multiple sites in the same cms – unknown domain)
        domain: domain,
        // page recheck
        recheck: recheck,
        // full recrawl
        recrawl: recrawl,
        // remove ui
        clear: clear,
        siCheckFlatDom: siCheckFlatDom,
    };

})(window, Granite.$, window.siteimprove.logger);
window.siteimprove.aem.init();
